# this code is auto generated by tools/prefix_ta.py

from polars_ta.ta.momentum import APO as ts_APO  # noqa
from polars_ta.ta.momentum import AROON_aroondown as ts_AROON_aroondown  # noqa
from polars_ta.ta.momentum import AROON_aroonup as ts_AROON_aroonup  # noqa
from polars_ta.ta.momentum import MACD_macd as ts_MACD_macd  # noqa
from polars_ta.ta.momentum import MACD_macdhist as ts_MACD_macdhist  # noqa
from polars_ta.ta.momentum import MACD_macdsignal as ts_MACD_macdsignal  # noqa
from polars_ta.ta.momentum import MOM as ts_MOM  # noqa
from polars_ta.ta.momentum import PPO as ts_PPO  # noqa
from polars_ta.ta.momentum import ROC as ts_ROC  # noqa
from polars_ta.ta.momentum import ROCP as ts_ROCP  # noqa
from polars_ta.ta.momentum import ROCR as ts_ROCR  # noqa
from polars_ta.ta.momentum import ROCR100 as ts_ROCR100  # noqa
from polars_ta.ta.momentum import RSI as ts_RSI  # noqa
from polars_ta.ta.momentum import RSV as ts_RSV  # noqa
from polars_ta.ta.momentum import STOCHF_fastd as ts_STOCHF_fastd  # noqa
from polars_ta.ta.momentum import TRIX as ts_TRIX  # noqa
from polars_ta.ta.momentum import WILLR as ts_WILLR  # noqa
from polars_ta.ta.overlap import BBANDS_upperband as ts_BBANDS_upperband  # noqa
from polars_ta.ta.overlap import DEMA as ts_DEMA  # noqa
from polars_ta.ta.overlap import EMA as ts_EMA  # noqa
from polars_ta.ta.overlap import KAMA as ts_KAMA  # noqa
from polars_ta.ta.overlap import MIDPOINT as ts_MIDPOINT  # noqa
from polars_ta.ta.overlap import MIDPRICE as ts_MIDPRICE  # noqa
from polars_ta.ta.overlap import RMA as ts_RMA  # noqa
from polars_ta.ta.overlap import SMA as ts_SMA  # noqa
from polars_ta.ta.overlap import TEMA as ts_TEMA  # noqa
from polars_ta.ta.overlap import TRIMA as ts_TRIMA  # noqa
from polars_ta.ta.overlap import WMA as ts_WMA  # noqa
from polars_ta.ta.price import AVGPRICE  # noqa
from polars_ta.ta.price import MEDPRICE  # noqa
from polars_ta.ta.price import TYPPRICE  # noqa
from polars_ta.ta.price import WCLPRICE  # noqa
from polars_ta.ta.statistic import BETA as ts_BETA  # noqa
from polars_ta.ta.statistic import CORREL as ts_CORREL  # noqa
from polars_ta.ta.statistic import LINEARREG as ts_LINEARREG  # noqa
from polars_ta.ta.statistic import LINEARREG_ANGLE as ts_LINEARREG_ANGLE  # noqa
from polars_ta.ta.statistic import LINEARREG_INTERCEPT as ts_LINEARREG_INTERCEPT  # noqa
from polars_ta.ta.statistic import LINEARREG_SLOPE as ts_LINEARREG_SLOPE  # noqa
from polars_ta.ta.statistic import STDDEV as ts_STDDEV  # noqa
from polars_ta.ta.statistic import TSF as ts_TSF  # noqa
from polars_ta.ta.statistic import VAR as ts_VAR  # noqa
from polars_ta.ta.transform import ACOS  # noqa
from polars_ta.ta.transform import ASIN  # noqa
from polars_ta.ta.transform import ATAN  # noqa
from polars_ta.ta.transform import CEIL  # noqa
from polars_ta.ta.transform import COS  # noqa
from polars_ta.ta.transform import COSH  # noqa
from polars_ta.ta.transform import EXP  # noqa
from polars_ta.ta.transform import FLOOR  # noqa
from polars_ta.ta.transform import LN  # noqa
from polars_ta.ta.transform import LOG10  # noqa
from polars_ta.ta.transform import SIN  # noqa
from polars_ta.ta.transform import SINH  # noqa
from polars_ta.ta.transform import SQRT  # noqa
from polars_ta.ta.transform import TAN  # noqa
from polars_ta.ta.transform import TANH  # noqa
from polars_ta.ta.volatility import ATR as ts_ATR  # noqa
from polars_ta.ta.volatility import NATR as ts_NATR  # noqa
from polars_ta.ta.volatility import TRANGE as ts_TRANGE  # noqa
from polars_ta.ta.volume import AD as ts_AD  # noqa
from polars_ta.ta.volume import ADOSC as ts_ADOSC  # noqa
from polars_ta.ta.volume import OBV as ts_OBV  # noqa