# this code is auto generated by tools/prefix_talib.py
    
from polars_ta.talib import HT_DCPERIOD as ts_HT_DCPERIOD  # noqa
from polars_ta.talib import HT_DCPHASE as ts_HT_DCPHASE  # noqa
from polars_ta.talib import HT_PHASOR as ts_HT_PHASOR  # noqa
from polars_ta.talib import HT_SINE as ts_HT_SINE  # noqa
from polars_ta.talib import HT_TRENDMODE as ts_HT_TRENDMODE  # noqa
from polars_ta.talib import ADD  # noqa
from polars_ta.talib import DIV  # noqa
from polars_ta.talib import MAX  # noqa
from polars_ta.talib import MAXINDEX  # noqa
from polars_ta.talib import MIN  # noqa
from polars_ta.talib import MININDEX  # noqa
from polars_ta.talib import MINMAX  # noqa
from polars_ta.talib import MINMAXINDEX  # noqa
from polars_ta.talib import MULT  # noqa
from polars_ta.talib import SUB  # noqa
from polars_ta.talib import SUM  # noqa
from polars_ta.talib import ACOS  # noqa
from polars_ta.talib import ASIN  # noqa
from polars_ta.talib import ATAN  # noqa
from polars_ta.talib import CEIL  # noqa
from polars_ta.talib import COS  # noqa
from polars_ta.talib import COSH  # noqa
from polars_ta.talib import EXP  # noqa
from polars_ta.talib import FLOOR  # noqa
from polars_ta.talib import LN  # noqa
from polars_ta.talib import LOG10  # noqa
from polars_ta.talib import SIN  # noqa
from polars_ta.talib import SINH  # noqa
from polars_ta.talib import SQRT  # noqa
from polars_ta.talib import TAN  # noqa
from polars_ta.talib import TANH  # noqa
from polars_ta.talib import ADX as ts_ADX  # noqa
from polars_ta.talib import ADXR as ts_ADXR  # noqa
from polars_ta.talib import APO as ts_APO  # noqa
from polars_ta.talib import AROON as ts_AROON  # noqa
from polars_ta.talib import AROONOSC as ts_AROONOSC  # noqa
from polars_ta.talib import BOP as ts_BOP  # noqa
from polars_ta.talib import CCI as ts_CCI  # noqa
from polars_ta.talib import CMO as ts_CMO  # noqa
from polars_ta.talib import DX as ts_DX  # noqa
from polars_ta.talib import MACD as ts_MACD  # noqa
from polars_ta.talib import MACDEXT as ts_MACDEXT  # noqa
from polars_ta.talib import MACDFIX as ts_MACDFIX  # noqa
from polars_ta.talib import MFI as ts_MFI  # noqa
from polars_ta.talib import MINUS_DI as ts_MINUS_DI  # noqa
from polars_ta.talib import MINUS_DM as ts_MINUS_DM  # noqa
from polars_ta.talib import MOM as ts_MOM  # noqa
from polars_ta.talib import PLUS_DI as ts_PLUS_DI  # noqa
from polars_ta.talib import PLUS_DM as ts_PLUS_DM  # noqa
from polars_ta.talib import PPO as ts_PPO  # noqa
from polars_ta.talib import ROC as ts_ROC  # noqa
from polars_ta.talib import ROCP as ts_ROCP  # noqa
from polars_ta.talib import ROCR as ts_ROCR  # noqa
from polars_ta.talib import ROCR100 as ts_ROCR100  # noqa
from polars_ta.talib import RSI as ts_RSI  # noqa
from polars_ta.talib import STOCH as ts_STOCH  # noqa
from polars_ta.talib import STOCHF as ts_STOCHF  # noqa
from polars_ta.talib import STOCHRSI as ts_STOCHRSI  # noqa
from polars_ta.talib import TRIX as ts_TRIX  # noqa
from polars_ta.talib import ULTOSC as ts_ULTOSC  # noqa
from polars_ta.talib import WILLR as ts_WILLR  # noqa
from polars_ta.talib import BBANDS as ts_BBANDS  # noqa
from polars_ta.talib import DEMA as ts_DEMA  # noqa
from polars_ta.talib import EMA as ts_EMA  # noqa
from polars_ta.talib import HT_TRENDLINE as ts_HT_TRENDLINE  # noqa
from polars_ta.talib import KAMA as ts_KAMA  # noqa
from polars_ta.talib import MA as ts_MA  # noqa
from polars_ta.talib import MAMA as ts_MAMA  # noqa
from polars_ta.talib import MAVP as ts_MAVP  # noqa
from polars_ta.talib import MIDPOINT as ts_MIDPOINT  # noqa
from polars_ta.talib import MIDPRICE as ts_MIDPRICE  # noqa
from polars_ta.talib import SAR as ts_SAR  # noqa
from polars_ta.talib import SAREXT as ts_SAREXT  # noqa
from polars_ta.talib import SMA as ts_SMA  # noqa
from polars_ta.talib import T3 as ts_T3  # noqa
from polars_ta.talib import TEMA as ts_TEMA  # noqa
from polars_ta.talib import TRIMA as ts_TRIMA  # noqa
from polars_ta.talib import WMA as ts_WMA  # noqa
from polars_ta.talib import CDL2CROWS as ts_CDL2CROWS  # noqa
from polars_ta.talib import CDL3BLACKCROWS as ts_CDL3BLACKCROWS  # noqa
from polars_ta.talib import CDL3INSIDE as ts_CDL3INSIDE  # noqa
from polars_ta.talib import CDL3LINESTRIKE as ts_CDL3LINESTRIKE  # noqa
from polars_ta.talib import CDL3OUTSIDE as ts_CDL3OUTSIDE  # noqa
from polars_ta.talib import CDL3STARSINSOUTH as ts_CDL3STARSINSOUTH  # noqa
from polars_ta.talib import CDL3WHITESOLDIERS as ts_CDL3WHITESOLDIERS  # noqa
from polars_ta.talib import CDLABANDONEDBABY as ts_CDLABANDONEDBABY  # noqa
from polars_ta.talib import CDLADVANCEBLOCK as ts_CDLADVANCEBLOCK  # noqa
from polars_ta.talib import CDLBELTHOLD as ts_CDLBELTHOLD  # noqa
from polars_ta.talib import CDLBREAKAWAY as ts_CDLBREAKAWAY  # noqa
from polars_ta.talib import CDLCLOSINGMARUBOZU as ts_CDLCLOSINGMARUBOZU  # noqa
from polars_ta.talib import CDLCONCEALBABYSWALL as ts_CDLCONCEALBABYSWALL  # noqa
from polars_ta.talib import CDLCOUNTERATTACK as ts_CDLCOUNTERATTACK  # noqa
from polars_ta.talib import CDLDARKCLOUDCOVER as ts_CDLDARKCLOUDCOVER  # noqa
from polars_ta.talib import CDLDOJI as ts_CDLDOJI  # noqa
from polars_ta.talib import CDLDOJISTAR as ts_CDLDOJISTAR  # noqa
from polars_ta.talib import CDLDRAGONFLYDOJI as ts_CDLDRAGONFLYDOJI  # noqa
from polars_ta.talib import CDLENGULFING as ts_CDLENGULFING  # noqa
from polars_ta.talib import CDLEVENINGDOJISTAR as ts_CDLEVENINGDOJISTAR  # noqa
from polars_ta.talib import CDLEVENINGSTAR as ts_CDLEVENINGSTAR  # noqa
from polars_ta.talib import CDLGAPSIDESIDEWHITE as ts_CDLGAPSIDESIDEWHITE  # noqa
from polars_ta.talib import CDLGRAVESTONEDOJI as ts_CDLGRAVESTONEDOJI  # noqa
from polars_ta.talib import CDLHAMMER as ts_CDLHAMMER  # noqa
from polars_ta.talib import CDLHANGINGMAN as ts_CDLHANGINGMAN  # noqa
from polars_ta.talib import CDLHARAMI as ts_CDLHARAMI  # noqa
from polars_ta.talib import CDLHARAMICROSS as ts_CDLHARAMICROSS  # noqa
from polars_ta.talib import CDLHIGHWAVE as ts_CDLHIGHWAVE  # noqa
from polars_ta.talib import CDLHIKKAKE as ts_CDLHIKKAKE  # noqa
from polars_ta.talib import CDLHIKKAKEMOD as ts_CDLHIKKAKEMOD  # noqa
from polars_ta.talib import CDLHOMINGPIGEON as ts_CDLHOMINGPIGEON  # noqa
from polars_ta.talib import CDLIDENTICAL3CROWS as ts_CDLIDENTICAL3CROWS  # noqa
from polars_ta.talib import CDLINNECK as ts_CDLINNECK  # noqa
from polars_ta.talib import CDLINVERTEDHAMMER as ts_CDLINVERTEDHAMMER  # noqa
from polars_ta.talib import CDLKICKING as ts_CDLKICKING  # noqa
from polars_ta.talib import CDLKICKINGBYLENGTH as ts_CDLKICKINGBYLENGTH  # noqa
from polars_ta.talib import CDLLADDERBOTTOM as ts_CDLLADDERBOTTOM  # noqa
from polars_ta.talib import CDLLONGLEGGEDDOJI as ts_CDLLONGLEGGEDDOJI  # noqa
from polars_ta.talib import CDLLONGLINE as ts_CDLLONGLINE  # noqa
from polars_ta.talib import CDLMARUBOZU as ts_CDLMARUBOZU  # noqa
from polars_ta.talib import CDLMATCHINGLOW as ts_CDLMATCHINGLOW  # noqa
from polars_ta.talib import CDLMATHOLD as ts_CDLMATHOLD  # noqa
from polars_ta.talib import CDLMORNINGDOJISTAR as ts_CDLMORNINGDOJISTAR  # noqa
from polars_ta.talib import CDLMORNINGSTAR as ts_CDLMORNINGSTAR  # noqa
from polars_ta.talib import CDLONNECK as ts_CDLONNECK  # noqa
from polars_ta.talib import CDLPIERCING as ts_CDLPIERCING  # noqa
from polars_ta.talib import CDLRICKSHAWMAN as ts_CDLRICKSHAWMAN  # noqa
from polars_ta.talib import CDLRISEFALL3METHODS as ts_CDLRISEFALL3METHODS  # noqa
from polars_ta.talib import CDLSEPARATINGLINES as ts_CDLSEPARATINGLINES  # noqa
from polars_ta.talib import CDLSHOOTINGSTAR as ts_CDLSHOOTINGSTAR  # noqa
from polars_ta.talib import CDLSHORTLINE as ts_CDLSHORTLINE  # noqa
from polars_ta.talib import CDLSPINNINGTOP as ts_CDLSPINNINGTOP  # noqa
from polars_ta.talib import CDLSTALLEDPATTERN as ts_CDLSTALLEDPATTERN  # noqa
from polars_ta.talib import CDLSTICKSANDWICH as ts_CDLSTICKSANDWICH  # noqa
from polars_ta.talib import CDLTAKURI as ts_CDLTAKURI  # noqa
from polars_ta.talib import CDLTASUKIGAP as ts_CDLTASUKIGAP  # noqa
from polars_ta.talib import CDLTHRUSTING as ts_CDLTHRUSTING  # noqa
from polars_ta.talib import CDLTRISTAR as ts_CDLTRISTAR  # noqa
from polars_ta.talib import CDLUNIQUE3RIVER as ts_CDLUNIQUE3RIVER  # noqa
from polars_ta.talib import CDLUPSIDEGAP2CROWS as ts_CDLUPSIDEGAP2CROWS  # noqa
from polars_ta.talib import CDLXSIDEGAP3METHODS as ts_CDLXSIDEGAP3METHODS  # noqa
from polars_ta.talib import AVGPRICE  # noqa
from polars_ta.talib import MEDPRICE  # noqa
from polars_ta.talib import TYPPRICE  # noqa
from polars_ta.talib import WCLPRICE  # noqa
from polars_ta.talib import BETA as ts_BETA  # noqa
from polars_ta.talib import CORREL as ts_CORREL  # noqa
from polars_ta.talib import LINEARREG as ts_LINEARREG  # noqa
from polars_ta.talib import LINEARREG_ANGLE as ts_LINEARREG_ANGLE  # noqa
from polars_ta.talib import LINEARREG_INTERCEPT as ts_LINEARREG_INTERCEPT  # noqa
from polars_ta.talib import LINEARREG_SLOPE as ts_LINEARREG_SLOPE  # noqa
from polars_ta.talib import STDDEV as ts_STDDEV  # noqa
from polars_ta.talib import TSF as ts_TSF  # noqa
from polars_ta.talib import VAR as ts_VAR  # noqa
from polars_ta.talib import ATR as ts_ATR  # noqa
from polars_ta.talib import NATR as ts_NATR  # noqa
from polars_ta.talib import TRANGE as ts_TRANGE  # noqa
from polars_ta.talib import AD as ts_AD  # noqa
from polars_ta.talib import ADOSC as ts_ADOSC  # noqa
from polars_ta.talib import OBV as ts_OBV  # noqa